import { Component, ChangeDetectionStrategy, ViewEncapsulation, Renderer2, ContentChild, Input, Optional, Host, HostListener, NgModule } from '@angular/core';
import { Storage } from '@bravobit/bb-foundation/storage';
import { BehaviorSubject, of } from 'rxjs';
import { map } from 'rxjs/operators';
import { Patch, Platform, mixinDisabled } from '@bravobit/bb-foundation';
import { ElementsModule } from '@bravobit/bb-foundation/elements';
import { CommonModule } from '@angular/common';
import { RouterModule } from '@angular/router';

class BbDashboardSidebar {
    constructor(_storage) {
        this._storage = _storage;
        // Bindings.
        this.visibilityChanges = new BehaviorSubject(true);
        // State.
        this._state$ = new BehaviorSubject({});
        // Token.
        this._localStorageToken = 'bb-dashboard-sidebar-state';
    }
    ngOnInit() {
        // Retrieve the cache.
        this._cache = this._storage.get(this._localStorageToken) || {};
    }
    registerGroup(id) {
        // Get the old state.
        const oldState = this._state$.getValue();
        // Get the cache value so when it is undefined
        // we will default to true.
        const cacheValue = this._cache[id] === undefined ? true : this._cache[id];
        // Push the new state.
        this._state$.next(Object.assign(Object.assign({}, oldState), { [id]: cacheValue }));
        // Return the slice of the state that the group needs.
        return this._state$.pipe(map(state => state[id]));
    }
    unregisterGroup(id) {
        // Get the old groups.
        const oldState = this._state$.getValue();
        // Generate the new state without the id.
        const newState = Object.keys(oldState).reduce((object, key) => {
            if (key !== id) {
                object[key] = oldState[key];
            }
            return object;
        }, {});
        // Push the new state.
        this._state$.next(newState);
    }
    toggleGroup(id, newValue) {
        // Get the old state.
        const oldState = this._state$.getValue();
        // Create the new state.
        const newState = Object.assign(Object.assign({}, oldState), { [id]: newValue });
        // Push the new state.
        this._state$.next(newState);
        // Save in the cache.
        this._storage.set(this._localStorageToken, newState);
    }
    toggle() {
        const current = this.visibilityChanges.getValue();
        this.visibilityChanges.next(!current);
    }
    open() {
        this.visibilityChanges.next(true);
    }
    close() {
        this.visibilityChanges.next(false);
    }
}
BbDashboardSidebar.decorators = [
    { type: Component, args: [{
                selector: 'bb-dashboard-sidebar',
                template: "<!--\n    The optional header element.\n-->\n<ng-content select=\"header\"></ng-content>\n\n<!--\n    The scroll area that contains the\n    groups in the navigation.\n-->\n<div class=\"bb-dashboard-sidebar-scroll\">\n    <ng-content select=\"bb-dashboard-sidebar-group\"></ng-content>\n</div>\n\n<!--\n    The optional footer element.\n-->\n<ng-content select=\"footer\"></ng-content>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dashboard-sidebar',
                    '[class.visible]': 'visibilityChanges.getValue()',
                    'role': 'navigation'
                },
                styles: [".bb-dashboard-sidebar{background:#fff linear-gradient(180deg,#fff,#f2f2f2);border-right:1px solid rgba(0,0,0,.4);bottom:0;display:flex;flex-direction:column;height:100vh;height:calc(var(--vh, 1vh)*100);left:-256px;min-width:256px;position:fixed;top:0;transform:translateX(256px);transition:transform .25s cubic-bezier(0,0,.2,1);width:256px;will-change:transform;z-index:10}.bb-dashboard-sidebar.visible{transform:none}.bb-dashboard-sidebar-scroll{flex:1;overflow:auto}.bb-dashboard-sidebar>header{background-color:hsla(0,0%,100%,.5);border-bottom:1px solid #d8d8d8}.bb-dashboard-sidebar>footer{background-color:hsla(0,0%,100%,.5);border-top:1px solid #d8d8d8}@media only screen and (min-width:768px){.bb-dashboard-sidebar{left:0;margin-left:-256px;position:relative;transform:none;transition:margin-left .25s cubic-bezier(0,0,.2,1);z-index:auto}.bb-dashboard-sidebar.visible{margin-left:0}}"]
            },] }
];
BbDashboardSidebar.ctorParameters = () => [
    { type: Storage }
];

class BbDashboard {
    constructor(_patch, _platform, _renderer) {
        this._patch = _patch;
        this._platform = _platform;
        this._renderer = _renderer;
        // Inputs.
        this.backgroundColor = '#e7eaed';
        this.maxWidth = '1200px';
        this.padding = '20px';
    }
    ngOnInit() {
        // Set the vertical height for mobile devices.
        this._patch.mobileVerticalHeight();
        // Set the HTML class to prevent scrolling.
        return this.setHtmlClass();
    }
    ngOnDestroy() {
        // Remove the HTML class.
        this.setHtmlClass(true);
    }
    setHtmlClass(shouldRemove = false) {
        // Check if we are on a browser.
        if (!this._platform.isBrowser) {
            return;
        }
        // Define the class name.
        const className = 'bb-dashboard-html';
        // Add or remove the class.
        return shouldRemove
            ? this._renderer.removeClass(document.documentElement, className)
            : this._renderer.addClass(document.documentElement, className);
    }
}
BbDashboard.decorators = [
    { type: Component, args: [{
                selector: 'bb-dashboard',
                template: "<!--\n    The outlet for where the sidebar\n    will be placed (if provided).\n-->\n<ng-content select=\"bb-dashboard-sidebar\">\n</ng-content>\n\n<!--\n    The section containing the header\n    and the main content.\n-->\n<section [style.background-color]=\"backgroundColor\"\n         class=\"bb-dashboard-section\">\n    <!--\n        The outlet for where the header\n        will be placed (if provided).\n    -->\n    <ng-content select=\"bb-dashboard-header\">\n    </ng-content>\n\n    <!--\n        The outlet of the content that\n        will be provided by the router.\n    -->\n    <main [style.padding]=\"padding\"\n          class=\"bb-dashboard-scroll\">\n        <div [style.max-width]=\"maxWidth\"\n             class=\"bb-dashboard-container\">\n            <ng-content select=\"bb-dashboard-menu\"></ng-content>\n\n            <ng-content></ng-content>\n        </div>\n        <!--\n            The outlet for where the footer\n            will be placed (if provided).\n        -->\n        <ng-content select=\"bb-dashboard-footer\">\n        </ng-content>\n    </main>\n</section>\n\n<!--\n    The backdrop that will be visible when\n    the sidebar is opened (only on mobile).\n-->\n<div *ngIf=\"!!sidebar\"\n     [class.visible]=\"!(sidebar?.visibilityChanges | async)\"\n     (click)=\"sidebar?.toggle()\"\n     class=\"bb-dashboard-backdrop\">\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dashboard'
                },
                styles: [".bb-dashboard{display:flex;height:100vh;height:calc(var(--vh, 1vh)*100);overflow:hidden;position:relative}.bb-dashboard-section{height:100vh;height:calc(var(--vh, 1vh)*100)}.bb-dashboard-scroll,.bb-dashboard-section{display:flex;flex:1;flex-direction:column;overflow-x:hidden}.bb-dashboard-scroll{-webkit-overflow-scrolling:touch;overflow-y:scroll}.bb-dashboard-container{display:flex;flex-direction:column;margin:0 auto;width:100%}.bb-dashboard-backdrop{background-color:rgba(0,0,0,.5);bottom:0;left:0;opacity:0;pointer-events:none;position:absolute;right:0;top:0;transition:opacity .2s cubic-bezier(0,0,.2,1);will-change:opacity;z-index:5}.bb-dashboard-backdrop.visible{cursor:pointer;opacity:1;pointer-events:all}@media only screen and (min-width:768px){.bb-dashboard-backdrop{display:none}}.bb-dashboard-html{height:100%;overflow:hidden}"]
            },] }
];
BbDashboard.ctorParameters = () => [
    { type: Patch },
    { type: Platform },
    { type: Renderer2 }
];
BbDashboard.propDecorators = {
    sidebar: [{ type: ContentChild, args: [BbDashboardSidebar,] }],
    backgroundColor: [{ type: Input }],
    maxWidth: [{ type: Input }],
    padding: [{ type: Input }]
};

class BbDashboardHeader {
    constructor(_parent) {
        this._parent = _parent;
    }
    get showSidebarButton() {
        var _a;
        return !!((_a = this._parent) === null || _a === void 0 ? void 0 : _a.sidebar);
    }
    get isSidebarVisible() {
        var _a, _b, _c;
        return (_c = (_b = (_a = this._parent) === null || _a === void 0 ? void 0 : _a.sidebar) === null || _b === void 0 ? void 0 : _b.visibilityChanges) !== null && _c !== void 0 ? _c : of(false);
    }
    onToggleSidebarClicked() {
        var _a, _b;
        (_b = (_a = this._parent) === null || _a === void 0 ? void 0 : _a.sidebar) === null || _b === void 0 ? void 0 : _b.toggle();
    }
}
BbDashboardHeader.decorators = [
    { type: Component, args: [{
                selector: 'bb-dashboard-header',
                template: "<!--\n    The button that toggles the sidebar.\n-->\n<button *ngIf=\"showSidebarButton\"\n        (click)=\"onToggleSidebarClicked()\"\n        [class.reversed]=\"isSidebarVisible | async\"\n        class=\"bb-dashboard-header-button\"\n        type=\"button\"\n        tabindex=\"0\"\n        role=\"switch\">\n    <svg xmlns=\"http://www.w3.org/2000/svg\"\n         viewBox=\"0 0 157 109\"\n         width=\"17\"\n         height=\"12\"\n         role=\"img\">\n        <title>\n            Toggle\n        </title>\n        <path fill=\"#fff\"\n              d=\"M82 42V11.2L30 54.5l52 43.3V67h75V42zM9.5 109h-2c-4.1 0-7.5-3.4-7.5-7.5v-94C0 3.4 3.4 0 7.5 0h2C13.6 0 17 3.4 17 7.5v94c0 4.1-3.4 7.5-7.5 7.5z\">\n        </path>\n    </svg>\n</button>\n\n<!--\n    The extra items of the header.\n-->\n<div class=\"bb-dashboard-header-items\">\n    <ng-content></ng-content>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dashboard-header'
                },
                styles: [".bb-dashboard-header{align-items:center;background-color:#5b53ff;border-bottom:1px solid #4d46d6;color:#fff;display:flex;height:50px;min-height:50px;padding:0 8px;width:100%;z-index:1}.bb-dashboard-header-button{align-items:center;background-color:transparent;border:1px solid #4d46d6;border-radius:4px;display:inline-flex;height:34px;justify-content:center;width:68px}.bb-dashboard-header-button>svg{height:100%;transform:none;transition:transform .2s cubic-bezier(0,0,.2,1)}.bb-dashboard-header-button.reversed>svg{transform:rotateY(180deg)}.bb-dashboard-header-button:hover{background-color:rgba(0,0,0,.04)}.bb-dashboard-header-button:active{background-color:rgba(0,0,0,.08)}.bb-dashboard-header-items{align-items:center;display:flex;margin-left:auto}.bb-dashboard-header-items>:not(:first-child){margin-left:8px}@media only screen and (min-width:768px){.bb-dashboard-header-button>svg{transform:rotateY(180deg)}.bb-dashboard-header-button.reversed>svg{transform:none}}"]
            },] }
];
BbDashboardHeader.ctorParameters = () => [
    { type: BbDashboard, decorators: [{ type: Optional }] }
];

class BbDashboardMenu {
    constructor() {
        // Inputs.
        this.title = null;
        this.imageUrl = null;
        this.backUrlLink = null;
        this.backUrlTitle = null;
    }
}
BbDashboardMenu.decorators = [
    { type: Component, args: [{
                selector: 'bb-dashboard-menu',
                template: "<a *ngIf=\"backUrlLink && backUrlTitle\"\n   [routerLink]=\"backUrlLink\"\n   class=\"bb-dashboard-menu-link\">\n    {{ backUrlTitle }}\n</a>\n\n<div class=\"bb-dashboard-menu-row\">\n    <img *ngIf=\"imageUrl\"\n         [src]=\"imageUrl\"\n         class=\"bb-dashboard-menu-image\"\n         alt=\"Menu image\">\n    <h1 *ngIf=\"title as titleLabel\"\n        class=\"bb-dashboard-menu-title\">\n        {{ titleLabel }}\n    </h1>\n</div>\n\n<nav class=\"bb-dashboard-menu-navigation\">\n    <ng-content select=\"[bb-dashboard-menu-item]\"></ng-content>\n</nav>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dashboard-menu',
                    'role': 'navigation'
                },
                styles: [".bb-dashboard-menu{display:block;margin-bottom:20px}.bb-dashboard-menu-link,.bb-dashboard-menu-title{max-width:100%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.bb-dashboard-menu-link{display:block;font-weight:500;margin-bottom:16px;text-decoration:none}.bb-dashboard-menu-link,.bb-dashboard-menu-link:visited{color:#a7a9c4!important}.bb-dashboard-menu-link:before{background-image:url(\"data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 492 492'%3E%3Cpath fill='%23a7a9c4' d='M464.344 207.418l.768.168H135.888l103.496-103.724c5.068-5.064 7.848-11.924 7.848-19.124 0-7.2-2.78-14.012-7.848-19.088L223.28 49.538c-5.064-5.064-11.812-7.864-19.008-7.864-7.2 0-13.952 2.78-19.016 7.844L7.844 226.914C2.76 231.998-.02 238.77 0 245.974c-.02 7.244 2.76 14.02 7.844 19.096l177.412 177.412c5.064 5.06 11.812 7.844 19.016 7.844 7.196 0 13.944-2.788 19.008-7.844l16.104-16.112c5.068-5.056 7.848-11.808 7.848-19.008 0-7.196-2.78-13.592-7.848-18.652L134.72 284.406h329.992c14.828 0 27.288-12.78 27.288-27.6v-22.788c0-14.82-12.828-26.6-27.656-26.6z'/%3E%3C/svg%3E\");background-size:cover;content:\"\";display:inline-block;height:13px;margin-right:3px;width:14px}.bb-dashboard-menu-row{align-items:center;display:flex;margin-bottom:10px}.bb-dashboard-menu-image{border-radius:50%;display:block;height:30px;margin-right:10px;width:30px}.bb-dashboard-menu-title{color:#272838;display:block;font-size:34px;font-weight:500;line-height:1}.bb-dashboard-menu-navigation{border-bottom:1px solid #d8d8d8;display:flex;overflow-x:auto}"]
            },] }
];
BbDashboardMenu.propDecorators = {
    title: [{ type: Input }],
    imageUrl: [{ type: Input }],
    backUrlLink: [{ type: Input }],
    backUrlTitle: [{ type: Input }]
};

class BbDashboardMenuItemBase {
}
const BbDashboardMenuItemMixinBase = mixinDisabled(BbDashboardMenuItemBase);
class BbDashboardMenuItem extends BbDashboardMenuItemMixinBase {
    stopDisabledEvents(event) {
        // If the menu item is not disabled let all events pass.
        if (!this.disabled) {
            return;
        }
        // If the menu item is disabled stop the event from
        // bubbling up.
        event.preventDefault();
        event.stopImmediatePropagation();
    }
}
BbDashboardMenuItem.decorators = [
    { type: Component, args: [{
                selector: '[bb-dashboard-menu-item]',
                template: "<ng-content select=\"[bbPrefix]\"></ng-content>\n<span class=\"bb-dashboard-menu-item-content\">\n    <ng-content></ng-content>\n</span>\n<ng-content select=\"[bbSuffix]\"></ng-content>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                inputs: ['disabled'],
                host: {
                    'class': 'bb-dashboard-menu-item',
                    '[class.disabled]': 'disabled',
                    '(click)': 'stopDisabledEvents($event)'
                },
                styles: [".bb-dashboard-menu-item{-moz-appearance:none;-webkit-appearance:none;align-items:center;appearance:none;background-color:transparent;border:none;border-bottom:2px solid transparent;display:flex;font-size:16px;margin:0;max-width:100%;padding:0;text-decoration:none!important;white-space:nowrap}.bb-dashboard-menu-item,.bb-dashboard-menu-item:visited{color:#6b5c5c}.bb-dashboard-menu-item:not(:first-child){margin-left:10px}.bb-dashboard-menu-item:not(:last-child){margin-right:10px}.bb-dashboard-menu-item:hover:not(.disabled):not(.active){border-color:rgba(117,119,148,.2)}.bb-dashboard-menu-item.active,.bb-dashboard-menu-item.active:visited .bb-dashboard-menu-item.active:hover{border-color:#5b53ff;color:#5b53ff}.bb-dashboard-menu-item.disabled,.bb-dashboard-menu-item.disabled:active,.bb-dashboard-menu-item.disabled:focus,.bb-dashboard-menu-item.disabled:hover{box-shadow:none;cursor:default;opacity:.4}.bb-dashboard-menu-item>.bb-prefix{margin-right:5px}.bb-dashboard-menu-item>.bb-suffix{margin-left:5px}.bb-dashboard-menu-item-content{display:block;font-weight:500;height:38px;line-height:38px}"]
            },] }
];

class BbDashboardSidebarGroupBase {
    constructor(_parent) {
        this._parent = _parent;
    }
}
const BbDashboardSidebarGroupMixinBase = mixinDisabled(BbDashboardSidebarGroupBase);
class BbDashboardSidebarGroup extends BbDashboardSidebarGroupMixinBase {
    constructor(_parent) {
        super(_parent);
        this._parent = _parent;
        // Inputs.
        this.id = null;
        this.icon = null;
        this.title = null;
        // State.
        this._isExpanded = false;
    }
    get hasToggle() {
        // Check if the user provided a icon and/or title.
        return !!this.icon || !!this.title;
    }
    get isExpanded() {
        // If there is an icon or title the group can be
        // expanded by clicking, else not.
        return this.hasToggle
            ? this._isExpanded
            : true;
    }
    ngOnInit() {
        var _a, _b;
        // Check if the id has been supplied.
        if (!this.id) {
            throw new Error('Id is required.');
        }
        // Register the group at the parent and
        // retrieve the slice of the state.
        const observable$ = (_b = (_a = this._parent) === null || _a === void 0 ? void 0 : _a.registerGroup(this.id)) !== null && _b !== void 0 ? _b : of(false);
        // Subscribe to the state.
        this._subscription = observable$
            .subscribe(isExpanded => this._isExpanded = isExpanded);
    }
    ngOnDestroy() {
        // Unsubscribe to prevent memory leaks.
        this._subscription && this._subscription.unsubscribe();
        // Unregister the group at the parent.
        this._parent && this._parent.unregisterGroup(this.id);
    }
    onClick() {
        var _a;
        // Verify the group is not disabled.
        if (this.disabled) {
            return;
        }
        // Change the state.
        const newValue = !this._isExpanded;
        // Set the state of the sidebar group.
        (_a = this._parent) === null || _a === void 0 ? void 0 : _a.toggleGroup(this.id, newValue);
    }
    onGroupClicked(event) {
        // If the group is not disabled don't do anything.
        if (!this.disabled) {
            return;
        }
        event.preventDefault();
        event.stopPropagation();
    }
}
BbDashboardSidebarGroup.decorators = [
    { type: Component, args: [{
                selector: 'bb-dashboard-sidebar-group',
                template: "<!--\n    The wrapper that contains the title and icon.\n-->\n<button *ngIf=\"hasToggle\"\n        (click)=\"onClick()\"\n        class=\"bb-dashboard-sidebar-group-title\"\n        type=\"button\"\n        role=\"switch\">\n    <bb-icon *ngIf=\"icon as iconName\"\n             [src]=\"iconName\"\n             class=\"bb-dashboard-sidebar-group-icon\">\n    </bb-icon>\n    <span *ngIf=\"title as titleText\"\n          role=\"heading\"\n          class=\"bb-dashboard-sidebar-group-text\">\n        {{ titleText }}\n    </span>\n    <span class=\"bb-dashboard-sidebar-group-toggle\">\n       <svg xmlns=\"http://www.w3.org/2000/svg\"\n            viewBox=\"0 0 21.7 30.9\"\n            width=\"10\"\n            height=\"10\"\n            role=\"img\">\n           <path\n               d=\"M10.1.3L.2 11.8c-.6.6-.1 1.7.8 1.7h19.7c.9 0 1.3-1 .8-1.7L11.6.3c-.4-.4-1.1-.4-1.5 0zM10.1 30.6L.2 19.1c-.6-.6-.1-1.7.8-1.7h19.7c.9 0 1.3 1 .8 1.7l-9.8 11.5c-.5.4-1.2.4-1.6 0z\">\n           </path>\n       </svg>\n    </span>\n</button>\n\n<!--\n    The sidebar items that contain the\n    links to navigate to other pages.\n-->\n<ng-content select=\"[bb-dashboard-sidebar-item]\">\n</ng-content>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                inputs: ['disabled'],
                host: {
                    'class': 'bb-dashboard-sidebar-group',
                    '[class.expanded]': 'isExpanded',
                    '[class.with-toggle]': 'hasToggle',
                    '[class.has-icon]': '!!icon',
                    '[class.disabled]': 'disabled',
                    'role': 'group'
                },
                styles: [".bb-dashboard-sidebar-group{border-bottom:1px solid rgba(0,0,0,.1);color:#000;display:block;overflow:hidden;transition:background-color .2s cubic-bezier(0,0,.2,1),padding-bottom .2s cubic-bezier(0,0,.2,1)}.bb-dashboard-sidebar-group.expanded{background-color:rgba(0,0,0,.04)}.bb-dashboard-sidebar-group.expanded .bb-dashboard-sidebar-group-title{color:#000}.bb-dashboard-sidebar-group.expanded .bb-dashboard-sidebar-item{height:32px;line-height:32px;opacity:1;padding:6px 16px}.bb-dashboard-sidebar-group.expanded .bb-dashboard-sidebar-group-toggle{opacity:0;transform:translateX(10px)}.bb-dashboard-sidebar-group.with-toggle.expanded{padding-bottom:20px}.bb-dashboard-sidebar-group.has-icon .bb-dashboard-sidebar-item{padding-left:54px}.bb-dashboard-sidebar-group.disabled{opacity:.5}.bb-dashboard-sidebar-group-title{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;align-items:center;background-color:transparent;border:none;color:rgba(0,0,0,.8);cursor:pointer;display:flex;margin:0;padding:17px 15px;text-align:left;transition:color .2s cubic-bezier(0,0,.2,1);user-select:none;width:100%}.bb-dashboard-sidebar-group-title:active>.bb-dashboard-sidebar-group-toggle{background-color:rgba(0,0,0,.08)}.bb-dashboard-sidebar-group-icon{color:rgba(0,0,0,.25);margin-right:15px}.bb-dashboard-sidebar-group-text{display:block;font-size:14px;font-weight:500;line-height:15px;max-width:100%;overflow:hidden;text-overflow:ellipsis;text-transform:uppercase;white-space:nowrap;width:100%}.bb-dashboard-sidebar-group-toggle{align-items:center;border-radius:3px;display:flex;height:22px;justify-content:center;margin-left:auto;min-height:22px;min-width:22px;transition:opacity .2s cubic-bezier(0,0,.2,1),transform .2s cubic-bezier(0,0,.2,1),background-color .2s cubic-bezier(0,0,.2,1);width:22px}.bb-dashboard-sidebar-group-toggle>svg{fill:rgba(0,0,0,.25);height:10px;width:10px}"]
            },] }
];
BbDashboardSidebarGroup.ctorParameters = () => [
    { type: BbDashboardSidebar, decorators: [{ type: Optional }, { type: Host }] }
];
BbDashboardSidebarGroup.propDecorators = {
    id: [{ type: Input }],
    icon: [{ type: Input }],
    title: [{ type: Input }],
    onGroupClicked: [{ type: HostListener, args: ['click', ['$event'],] }]
};

class BbDashboardSidebarItem {
    constructor(_sidebar, _group) {
        this._sidebar = _sidebar;
        this._group = _group;
        // Inputs.
        this.tabIndex = 0;
    }
    onClick() {
        var _a;
        // Try to "open" the sidebar if it exists.
        // This will close it on mobile.
        (_a = this._sidebar) === null || _a === void 0 ? void 0 : _a.open();
    }
    get attributeTabIndex() {
        // Make the items tabindex -1 so it can't
        // be tabbed to when the parent is not visible
        // or the parent is disabled.
        if (this.isHidden) {
            return -1;
        }
        // Else return the specified tabindex.
        return this.tabIndex;
    }
    get isHidden() {
        return !this.isVisible || this.isDisabled;
    }
    get isVisible() {
        return this._group
            ? this._group.isExpanded
            : true;
    }
    get isDisabled() {
        return this._group
            ? this._group.disabled
            : false;
    }
}
BbDashboardSidebarItem.decorators = [
    { type: Component, args: [{
                selector: '[bb-dashboard-sidebar-item]',
                template: "<!--\n    The content between the tags will\n    be reflected here.\n-->\n<span class=\"bb-dashboard-sidebar-item-content\">\n    <ng-content></ng-content>\n</span>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dashboard-sidebar-item',
                    '[attr.tabindex]': 'attributeTabIndex',
                    '[attr.aria-hidden]': 'isHidden',
                    '(click)': 'onClick()'
                },
                styles: [".bb-dashboard-sidebar-item{align-items:center;color:rgba(0,0,0,.85)!important;cursor:pointer;display:flex;font-size:14px;font-weight:500;height:0;line-height:0;opacity:0;padding:0;text-decoration:none;transition:height .2s cubic-bezier(0,0,.2,1),opacity .2s cubic-bezier(0,0,.2,1) 50ms,color .2s cubic-bezier(0,0,.2,1),background-color .2s cubic-bezier(0,0,.2,1);will-change:opacity,height}.bb-dashboard-sidebar-item:focus,.bb-dashboard-sidebar-item:hover{background-color:rgba(0,0,0,.02)}.bb-dashboard-sidebar-item:active{background-color:rgba(0,0,0,.04)}.bb-dashboard-sidebar-item.active,.bb-dashboard-sidebar-item.active:active,.bb-dashboard-sidebar-item.active:focus,.bb-dashboard-sidebar-item.active:hover{background-color:rgba(0,0,0,.06);color:#000!important}.bb-dashboard-sidebar-item-content{max-width:100%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}"]
            },] }
];
BbDashboardSidebarItem.ctorParameters = () => [
    { type: BbDashboardSidebar, decorators: [{ type: Optional }, { type: Host }] },
    { type: BbDashboardSidebarGroup, decorators: [{ type: Optional }, { type: Host }] }
];
BbDashboardSidebarItem.propDecorators = {
    tabIndex: [{ type: Input }]
};

const DECLARATIONS_EXPORTS = [
    BbDashboard,
    BbDashboardHeader,
    BbDashboardMenu,
    BbDashboardMenuItem,
    BbDashboardSidebar,
    BbDashboardSidebarGroup,
    BbDashboardSidebarItem
];
class DashboardModule {
}
DashboardModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, RouterModule, ElementsModule],
                declarations: [...DECLARATIONS_EXPORTS],
                exports: [...DECLARATIONS_EXPORTS]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { BbDashboard, BbDashboardHeader, BbDashboardMenu, BbDashboardMenuItem, BbDashboardSidebar, BbDashboardSidebarGroup, BbDashboardSidebarItem, DashboardModule };
//# sourceMappingURL=bravobit-bb-foundation-dashboard.js.map
