import { take, switchMap } from 'rxjs/operators';
import { Subject, fromEvent, race } from 'rxjs';
import { EventEmitter, Component, ChangeDetectionStrategy, ViewEncapsulation, ElementRef, Input, Output, HostListener, HostBinding, Directive, ViewContainerRef, ChangeDetectorRef, ComponentFactoryResolver, ViewChild, Injectable, Injector, ApplicationRef, NgModule } from '@angular/core';
import { trigger, transition, group, query, style, animate } from '@angular/animations';
import { Patch } from '@bravobit/bb-foundation';
import { BbButton, ElementsModule } from '@bravobit/bb-foundation/elements';
import { LocalizeModule } from '@bravobit/bb-foundation/localize';
import { CommonModule } from '@angular/common';

class BbDialogRef {
    constructor() {
        // Data.
        this._afterClosed$ = new Subject();
        this._overlayClicked$ = new Subject();
    }
    get afterClosed() {
        return this._afterClosed$.pipe(take(1));
    }
    get overlayClicked() {
        return this._overlayClicked$.pipe(take(1));
    }
    onOverlayClicked() {
        this._overlayClicked$.next();
    }
    close(result) {
        this._afterClosed$.next(result);
    }
}

class BbDialogConfig {
}

class BbDialogOverlay {
    constructor(_elementRef) {
        this._elementRef = _elementRef;
        // Inputs.
        this.animations = true;
        // Outputs.
        this.closeRequested = new EventEmitter();
        // State.
        this._state = 'open';
        this._closed$ = new Subject();
    }
    get state() {
        return this.animations
            ? this._state
            : 'none';
    }
    ngAfterViewInit() {
        this.listenForClicks();
    }
    ngOnDestroy() {
        this._subscription && this._subscription.unsubscribe();
    }
    onKeyEvent(event) {
        if (event.key === 'Escape') {
            this.closeRequested.emit();
        }
    }
    onAnimationDone(event) {
        // Check if the dialog is being destroyed.
        if (!['close', 'none'].includes(event.toState)) {
            return;
        }
        this._closed$.next();
        this._closed$.complete();
    }
    close() {
        this._state = 'close';
        return this._closed$.pipe(take(1)).toPromise();
    }
    listenForClicks() {
        if (!this._elementRef || !this._elementRef.nativeElement) {
            return;
        }
        const mouseDown$ = fromEvent(this._elementRef.nativeElement, 'mousedown');
        const touchStart$ = fromEvent(this._elementRef.nativeElement, 'touchstart');
        const start$ = race([mouseDown$, touchStart$]);
        const mouseUp$ = fromEvent(this._elementRef.nativeElement, 'mouseup');
        const touchEnd$ = fromEvent(this._elementRef.nativeElement, 'touchend');
        const end$ = race([mouseUp$, touchEnd$]);
        this._subscription = start$.pipe(switchMap(() => end$)).subscribe(() => {
            this.closeRequested.next();
        });
    }
}
BbDialogOverlay.decorators = [
    { type: Component, args: [{
                selector: 'bb-dialog-overlay',
                template: "<div class=\"bb-dialog-overlay-backdrop\"></div>\n\n<ng-content select=\"[bb-dialog-modal]\"></ng-content>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: { 'class': 'bb-dialog-overlay', '[@bbDialogAnimation]': 'state' },
                animations: [
                    trigger('bbDialogAnimation', [
                        transition('* => none', []),
                        transition(':enter, * => open', [
                            group([
                                query('.bb-dialog-overlay-backdrop', [
                                    style({ opacity: 0 }),
                                    animate('150ms cubic-bezier(0, 0, 0.2, 1)', style({ opacity: 1 }))
                                ]),
                                query('.bb-dialog-modal', [
                                    style({ transform: 'scale(0.95)' }),
                                    animate('150ms cubic-bezier(0, 0, 0.2, 1)', style({ transform: 'scale(1)' }))
                                ])
                            ])
                        ]),
                        transition(':leave, * => close', [
                            group([
                                query('.bb-dialog-overlay-backdrop', [
                                    animate('150ms cubic-bezier(0, 0, 0.2, 1)', style({ opacity: 0 }))
                                ]),
                                query('.bb-dialog-modal', [
                                    animate('150ms cubic-bezier(0, 0, 0.2, 1)', style({ opacity: 0, transform: 'scale(0.95)' }))
                                ])
                            ])
                        ])
                    ])
                ],
                styles: [".bb-dialog-overlay,.bb-dialog-overlay-backdrop{bottom:0;display:flex;flex-direction:column;left:0;position:fixed;right:0;top:0;z-index:1000}.bb-dialog-overlay{padding:20px}.bb-dialog-overlay-backdrop{background-color:rgba(0,0,0,.5)}.bb-dialog-overlay-component{display:block;position:relative;z-index:1001}"]
            },] }
];
BbDialogOverlay.ctorParameters = () => [
    { type: ElementRef }
];
BbDialogOverlay.propDecorators = {
    animations: [{ type: Input }],
    closeRequested: [{ type: Output }],
    onKeyEvent: [{ type: HostListener, args: ['window:keyup', ['$event'],] }],
    onAnimationDone: [{ type: HostListener, args: ['@bbDialogAnimation.done', ['$event'],] }]
};

class BbDialogModal {
    constructor(_patch) {
        this._patch = _patch;
        // Styling.
        this.maxWidth = '420px';
    }
    onOverlayClickedEvent(event) {
        event.stopPropagation();
    }
    ngOnInit() {
        this._patch.mobileVerticalHeight();
    }
}
BbDialogModal.decorators = [
    { type: Component, args: [{
                selector: '[bb-dialog-modal]',
                template: "<ng-content select=\"[bb-dialog-header]\">\n\n</ng-content>\n\n<div class=\"bb-dialog-modal-body\">\n    <ng-content></ng-content>\n</div>\n\n<ng-content select=\"[bb-dialog-actions]\">\n\n</ng-content>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dialog-modal',
                    'role': 'dialog'
                },
                styles: [".bb-dialog-modal{background-color:#fff;border-radius:4px;display:flex;flex-direction:column;margin:auto;max-height:calc(var(--vh, 1vh)*100 - 40px);max-width:420px;position:relative;width:100%;z-index:1001}.bb-dialog-modal-body{background-color:#f1f3f6;color:#738694;font-size:13px;line-height:1.4;overflow-y:auto;padding:20px}"]
            },] }
];
BbDialogModal.ctorParameters = () => [
    { type: Patch }
];
BbDialogModal.propDecorators = {
    maxWidth: [{ type: HostBinding, args: ['style.max-width',] }, { type: Input }],
    onOverlayClickedEvent: [{ type: HostListener, args: ['click', ['$event'],] }, { type: HostListener, args: ['touchstart', ['$event'],] }, { type: HostListener, args: ['touchend', ['$event'],] }, { type: HostListener, args: ['mouseup', ['$event'],] }, { type: HostListener, args: ['mousedown', ['$event'],] }]
};

class BbDialogHeader {
    constructor() {
        // Inputs.
        this.iconName = null;
        this.iconColor = 'black';
        // Outputs.
        this.closeRequested = new EventEmitter();
    }
}
BbDialogHeader.decorators = [
    { type: Component, args: [{
                selector: '[bb-dialog-header]',
                template: "<div class=\"bb-dialog-header-row\">\n    <div *ngIf=\"iconName as icon\"\n         [style.color]=\"iconColor\"\n         class=\"bb-dialog-header-icon\">\n        <i class=\"material-icons\">{{ icon }}</i>\n    </div>\n\n    <div class=\"bb-dialog-header-content\">\n        <ng-content></ng-content>\n    </div>\n\n    <button *ngIf=\"closeRequested?.observers?.length > 0\"\n            (click)=\"closeRequested?.emit()\"\n            type=\"button\"\n            class=\"bb-dialog-header-close\">\n    </button>\n</div>\n\n<nav class=\"bb-dialog-header-navigation\">\n    <ng-content select=\"[bb-dialog-link]\"></ng-content>\n</nav>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: { 'class': 'bb-dialog-header' },
                styles: [".bb-dialog-header{background-color:#fff;border-bottom:1px solid #d8d8d8;border-top-left-radius:4px;border-top-right-radius:4px;display:flex;flex-direction:column;justify-content:center;min-height:60px}.bb-dialog-header .bb-dialog-header-content,.bb-dialog-header .bb-dialog-header-content h1{font-size:20px;font-weight:500;max-width:100%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.bb-dialog-header-navigation,.bb-dialog-header-row{align-items:center;display:flex;width:100%}.bb-dialog-header-row{padding:10px 20px}.bb-dialog-header-navigation{overflow-x:auto;padding:0 20px;white-space:nowrap;width:100%}.bb-dialog-header-navigation:empty{height:0;margin:0;padding:0;visibility:hidden}.bb-dialog-header-content{flex:1}.bb-dialog-header-icon{align-items:center;border-radius:50%;display:flex;height:40px;justify-content:center;margin-right:10px;min-height:40px;min-width:40px;overflow:hidden;position:relative;width:40px}.bb-dialog-header-icon:after{background-color:currentColor;bottom:0;content:\"\";left:0;opacity:.14;position:absolute;right:0;top:0}.bb-dialog-header-close{background-color:transparent;border:none;border-radius:4px;display:inline-block;height:32px;margin-left:10px;margin-right:-10px;min-height:32px;min-width:32px;overflow:hidden;position:relative;width:32px}.bb-dialog-header-close:after,.bb-dialog-header-close:before{background-color:#a2a9ba;content:\"\";height:2px;left:20%;margin-top:-1px;position:absolute;top:50%;width:60%}.bb-dialog-header-close:before{transform:rotate(45deg)}.bb-dialog-header-close:after{transform:rotate(-45deg)}.bb-dialog-header-close:hover{background-color:rgba(0,0,0,.08)}.bb-dialog-header-close:focus{background-color:rgba(0,0,0,.12)}.bb-dialog-header-close:active{background-color:rgba(0,0,0,.16)}"]
            },] }
];
BbDialogHeader.propDecorators = {
    iconName: [{ type: Input }],
    iconColor: [{ type: Input }],
    closeRequested: [{ type: Output }]
};

class BbDialogLink {
}
BbDialogLink.decorators = [
    { type: Component, args: [{
                selector: '[bb-dialog-link]',
                template: "<ng-content></ng-content>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: { 'class': 'bb-dialog-link' },
                styles: [".bb-dialog-link{align-items:center;background-color:transparent;border-bottom:2px solid transparent;color:#6b5c5c!important;display:flex;font-size:16px;font-weight:500;height:30px;text-decoration:none!important;white-space:nowrap}.bb-dialog-link+.bb-dialog-link{margin-left:10px}.bb-dialog-link:hover:not(.disabled){border-bottom:2px solid rgba(117,119,148,.2)}.bb-dialog-link.active,.bb-dialog-link.active:hover{border-bottom-color:#5b53ff;color:#5b53ff!important}.bb-dialog-link.disabled,.bb-dialog-link.disabled:active,.bb-dialog-link.disabled:focus,.bb-dialog-link.disabled:hover{box-shadow:none;cursor:default;opacity:.4}"]
            },] }
];

class BbDialogActions {
}
BbDialogActions.decorators = [
    { type: Component, args: [{
                selector: '[bb-dialog-actions]',
                template: "<bb-form-group class=\"end\">\n    <ng-content></ng-content>\n</bb-form-group>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: { 'class': 'bb-dialog-actions' },
                styles: [".bb-dialog-actions{background-color:#fff;border-bottom-left-radius:4px;border-bottom-right-radius:4px;border-top:1px solid #d8d8d8;display:block;padding:20px}"]
            },] }
];

class BbDialogInsertion {
    constructor(viewContainerRef) {
        this.viewContainerRef = viewContainerRef;
    }
}
BbDialogInsertion.decorators = [
    { type: Directive, args: [{
                selector: '[bbDialogInsertion]'
            },] }
];
BbDialogInsertion.ctorParameters = () => [
    { type: ViewContainerRef }
];

class BbDialogContainer {
    constructor(_elementRef, _changeDetection, _componentFactoryResolver) {
        this._elementRef = _elementRef;
        this._changeDetection = _changeDetection;
        this._componentFactoryResolver = _componentFactoryResolver;
        // Data.
        this.componentRef = null;
        this.childComponentType = null;
        // State.
        this.overlayClicked$ = new Subject();
    }
    get onOverlayClicked() {
        return this.overlayClicked$.pipe(take(1));
    }
    ngAfterViewInit() {
        // Load the child component.
        this.loadChildComponent(this.childComponentType);
        // Listen for mouse click on the dialog container.
        this.listenForClicks();
        // Detect changes to notify change detection.
        this._changeDetection.detectChanges();
    }
    ngOnDestroy() {
        this._subscription && this._subscription.unsubscribe();
    }
    onAnimationDone(event) {
        // Check if the dialog is being destroyed.
        if (event.toState !== 'void') {
            return;
        }
        // Destroy the ref if it exists.
        this.componentRef && this.componentRef.destroy();
    }
    listenForClicks() {
        if (!this._elementRef || !this._elementRef.nativeElement) {
            return;
        }
        const mouseDown$ = fromEvent(this._elementRef.nativeElement, 'mousedown');
        const touchStart$ = fromEvent(this._elementRef.nativeElement, 'touchstart');
        const start$ = race([mouseDown$, touchStart$]);
        const mouseUp$ = fromEvent(this._elementRef.nativeElement, 'mouseup');
        const touchEnd$ = fromEvent(this._elementRef.nativeElement, 'touchend');
        const end$ = race([mouseUp$, touchEnd$]);
        this._subscription = start$.pipe(switchMap(() => end$)).subscribe(() => {
            this.overlayClicked$.next();
        });
    }
    loadChildComponent(componentType) {
        // Resolve the component.
        const componentFactory = this._componentFactoryResolver.resolveComponentFactory(componentType);
        // Get the view container ref and clear it.
        const viewContainerRef = this.insertion.viewContainerRef;
        viewContainerRef.clear();
        // Create an instance of the component.
        this.componentRef = viewContainerRef.createComponent(componentFactory);
    }
}
BbDialogContainer.decorators = [
    { type: Component, args: [{
                selector: 'bb-dialog-container',
                template: "<div class=\"bb-dialog-container-backdrop\">\n</div>\n\n<div class=\"bb-dialog-container-component\">\n    <ng-container bbDialogInsertion>\n    </ng-container>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                host: { 'class': 'bb-dialog-container', '[@bbDialogAnimation]': 'true' },
                preserveWhitespaces: false,
                animations: [
                    trigger('bbDialogAnimation', [
                        transition(':enter', [
                            group([
                                query('.bb-dialog-container-backdrop', [
                                    style({
                                        opacity: 0
                                    }),
                                    animate('150ms cubic-bezier(0, 0, 0.2, 1)', style({
                                        opacity: 1
                                    }))
                                ]),
                                query('.bb-dialog-container-component', [
                                    style({
                                        transform: 'scale(0.95)'
                                    }),
                                    animate('150ms cubic-bezier(0, 0, 0.2, 1)', style({
                                        transform: 'scale(1)'
                                    }))
                                ])
                            ])
                        ]),
                        transition(':leave', [
                            group([
                                query('.bb-dialog-container-backdrop', [
                                    animate('150ms cubic-bezier(0, 0, 0.2, 1)', style({
                                        opacity: 0
                                    }))
                                ]),
                                query('.bb-dialog-container-component', [
                                    animate('150ms cubic-bezier(0, 0, 0.2, 1)', style({
                                        opacity: 0,
                                        transform: 'scale(0.95)'
                                    }))
                                ])
                            ])
                        ])
                    ])
                ],
                styles: [".bb-dialog-container,.bb-dialog-container-backdrop{bottom:0;left:0;position:fixed;right:0;top:0;z-index:1000}.bb-dialog-container{display:flex}.bb-dialog-container-backdrop{background-color:rgba(0,0,0,.5)}.bb-dialog-container-component{display:block;margin:auto;padding:20px;width:100%;z-index:1001}"]
            },] }
];
BbDialogContainer.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: ComponentFactoryResolver }
];
BbDialogContainer.propDecorators = {
    insertion: [{ type: ViewChild, args: [BbDialogInsertion, { static: true },] }],
    onAnimationDone: [{ type: HostListener, args: ['@bbDialogAnimation.done', ['$event'],] }]
};

let nextUniqueId = 0;
class BbDialogConfirm {
    constructor(config, dialogRef) {
        this.config = config;
        this.dialogRef = dialogRef;
        // Id.
        this.dialogId = `bb-dialog-confirm-${nextUniqueId++}`;
    }
    ngAfterViewInit() {
        // Initial focus the button.
        this.confirmButton.focus();
    }
    onClose(result) {
        this.dialogRef.close(result);
    }
}
BbDialogConfirm.decorators = [
    { type: Component, args: [{
                selector: 'bb-dialog-confirm',
                template: "<div [attr.aria-labelledby]=\"dialogId\"\n     bb-dialog-modal\n     bbFocusTrap>\n    <header bb-dialog-header>\n        <h1 *ngIf=\"config?.data?.title as title\"\n            [id]=\"dialogId\">\n            {{ title | bbLocalize:{optional: true} }}\n        </h1>\n    </header>\n\n    <p>\n        {{ config?.data?.description | bbLocalize:{optional: true} }}\n    </p>\n\n    <footer bb-dialog-actions>\n        <button *ngIf=\"config?.data?.cancelButtonText as cancelButtonText\"\n                (click)=\"onClose(false)\"\n                bb-button\n                type=\"button\">\n            {{ cancelButtonText | bbLocalize:{optional: true} }}\n        </button>\n        <button #confirm\n                *ngIf=\"config?.data?.confirmButtonText as confirmButtonText\"\n                (click)=\"onClose(true)\"\n                bb-button\n                type=\"button\"\n                class=\"destructive\">\n            {{ confirmButtonText | bbLocalize:{optional: true} }}\n        </button>\n    </footer>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: { 'class': 'bb-dialog-confirm' },
                styles: [".bb-dialog-confirm{display:block}"]
            },] }
];
BbDialogConfirm.ctorParameters = () => [
    { type: BbDialogConfig },
    { type: BbDialogRef }
];
BbDialogConfirm.propDecorators = {
    confirmButton: [{ type: ViewChild, args: ['confirm', { read: BbButton },] }]
};

class BbDialogInjector {
    constructor(_parentInjector, _additionalTokens) {
        this._parentInjector = _parentInjector;
        this._additionalTokens = _additionalTokens;
    }
    get(token, notFoundValue) {
        // First check if the additional token
        // contains the token.
        const value = this._additionalTokens.get(token);
        // If that value exists return it.
        if (value) {
            return value;
        }
        // Else query the parent injector.
        return this._parentInjector.get(token, notFoundValue);
    }
}

class Dialog {
    constructor(_injector, _applicationRef, _componentFactoryResolver) {
        this._injector = _injector;
        this._applicationRef = _applicationRef;
        this._componentFactoryResolver = _componentFactoryResolver;
    }
    open(componentType, config) {
        return this.createDialog(componentType, config);
    }
    confirm(title, description, options = {}) {
        return this.open(BbDialogConfirm, {
            data: {
                title,
                description,
                cancelButtonText: options.cancelButtonText || 'dialog.default_cancel_button',
                confirmButtonText: options.confirmButtonText || 'dialog.default_confirm_button'
            }
        });
    }
    createDialog(componentType, config) {
        // Create a reference.
        const dialogRef = new BbDialogRef();
        // Create a map with the config.
        const map = new WeakMap();
        map.set(BbDialogConfig, config);
        map.set(BbDialogRef, dialogRef);
        // Create a custom injector.
        const dialogInjector = new BbDialogInjector(this._injector, map);
        // Resolve the dialog component reference.
        const dialogComponentRef = this._componentFactoryResolver
            .resolveComponentFactory(BbDialogContainer)
            .create(dialogInjector);
        // Attach the host view to the application.
        this._applicationRef.attachView(dialogComponentRef.hostView);
        try {
            // Remove the current focus.
            const activeElement = document.activeElement;
            activeElement.blur();
            // Append the element to the DOM.
            document.body.appendChild(dialogComponentRef.location.nativeElement);
        }
        catch (_a) {
            // Don't do anything, because it must've failed.
        }
        // Set the child component type on the component.
        dialogComponentRef.instance.childComponentType = componentType;
        dialogComponentRef.instance.onOverlayClicked.subscribe(() => dialogRef.onOverlayClicked());
        // Listen for close events.
        dialogRef.afterClosed.subscribe(() => this.deleteDialog(dialogComponentRef));
        // Return the reference.
        return dialogRef;
    }
    deleteDialog(dialogComponentRef) {
        // Only continue if the dialog component ref exists.
        if (!dialogComponentRef) {
            return;
        }
        // Detach the dialog.
        this._applicationRef.detachView(dialogComponentRef.hostView);
        // Destroy the dialog.
        dialogComponentRef.destroy();
        dialogComponentRef = null;
    }
}
Dialog.decorators = [
    { type: Injectable }
];
Dialog.ctorParameters = () => [
    { type: Injector },
    { type: ApplicationRef },
    { type: ComponentFactoryResolver }
];

class DialogModule {
    static forRoot() {
        return {
            ngModule: DialogModule,
            providers: [Dialog]
        };
    }
}
DialogModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    LocalizeModule,
                    ElementsModule
                ],
                declarations: [
                    BbDialogContainer,
                    BbDialogOverlay,
                    BbDialogModal,
                    BbDialogHeader,
                    BbDialogLink,
                    BbDialogActions,
                    BbDialogConfirm,
                    BbDialogInsertion
                ],
                entryComponents: [
                    BbDialogContainer,
                    BbDialogConfirm
                ],
                exports: [
                    BbDialogOverlay,
                    BbDialogModal,
                    BbDialogHeader,
                    BbDialogLink,
                    BbDialogActions
                ]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { BbDialogActions, BbDialogConfig, BbDialogHeader, BbDialogLink, BbDialogModal, BbDialogOverlay, BbDialogRef, Dialog, DialogModule, BbDialogContainer as ɵa, BbDialogInsertion as ɵb, BbDialogConfirm as ɵc };
//# sourceMappingURL=bravobit-bb-foundation-dialog.js.map
