import { InjectionToken, Injectable, RendererFactory2, Optional, Inject, EventEmitter, Component, ChangeDetectionStrategy, ViewEncapsulation, forwardRef, NgZone, ElementRef, HostBinding, Input, Output, NgModule } from '@angular/core';
import { Platform, mixinGrouped } from '@bravobit/bb-foundation';
import { DOCUMENT } from '@angular/common';
import { BehaviorSubject, Subscription } from 'rxjs';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

const RECAPTCHA_CONFIG = new InjectionToken('bb-recaptcha-config');

class RecaptchaLoader {
    constructor(_platform, _renderFactory, _document, _config) {
        this._platform = _platform;
        this._renderFactory = _renderFactory;
        this._document = _document;
        this._config = _config;
        // Ready state.
        this.ready$ = null;
        // Data.
        this._windowLoadFn = 'bbRecaptchaLoad';
        this._renderer = null;
        // Create a renderer.
        this._renderer = this._renderFactory.createRenderer(null, null);
        // Set the ready subject if not exists.
        if (this.ready$) {
            return;
        }
        this.ready$ = new BehaviorSubject(null);
        // Create the script.
        this.createScript(captcha => this.ready$.next(captcha));
    }
    createScript(callback) {
        var _a;
        // Validate we are on a browser.
        if (!this._platform.isBrowser) {
            return;
        }
        // Create the script.
        const script = this._renderer.createElement('script');
        // Set properties on the script.
        this._renderer.setProperty(script, 'src', this.getUrl());
        this._renderer.setProperty(script, 'async', true);
        this._renderer.setProperty(script, 'defer', true);
        // Set the nonce.
        if (this._config.nonce) {
            this._renderer.setAttribute(script, 'nonce', this._config.nonce);
        }
        // Set the loading function.
        if (this._platform.isBrowser) {
            window[this._windowLoadFn] = () => callback(grecaptcha);
        }
        // Append the script tag.
        this._renderer.appendChild((_a = this._document) === null || _a === void 0 ? void 0 : _a.body, script);
    }
    getUrl() {
        var _a, _b;
        const baseUrl = (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.baseUrl) !== null && _b !== void 0 ? _b : 'https://www.google.com/recaptcha/api.js';
        return [baseUrl, this.getQueryParams()]
            .filter(item => !!item)
            .join('?');
    }
    getQueryParams() {
        var _a, _b;
        const params = {
            render: 'explicit',
            onload: (_a = this._windowLoadFn) !== null && _a !== void 0 ? _a : null,
            hl: (_b = this._config.language) !== null && _b !== void 0 ? _b : null
        };
        return Object.keys(params)
            .map(key => ({ key, value: params[key] }))
            .filter(item => !!item.value)
            .map(item => `${item === null || item === void 0 ? void 0 : item.key}=${item === null || item === void 0 ? void 0 : item.value}`)
            .join('&');
    }
}
RecaptchaLoader.decorators = [
    { type: Injectable }
];
RecaptchaLoader.ctorParameters = () => [
    { type: Platform },
    { type: RendererFactory2 },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [DOCUMENT,] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [RECAPTCHA_CONFIG,] }] }
];

let nextUniqueId = 0;
class BbRecaptchaBase {
}
const BbRecaptchaMixinBase = mixinGrouped(BbRecaptchaBase);
class RecaptchaComponent extends BbRecaptchaMixinBase {
    constructor(_zone, _elementRef, _loader, _config) {
        super();
        this._zone = _zone;
        this._elementRef = _elementRef;
        this._loader = _loader;
        this._config = _config;
        // Host bindings.
        this.id = `bb-recaptcha-${nextUniqueId++}`;
        // Inputs.
        this.siteKey = null;
        this.tabIndex = null;
        this.type = null;
        this.size = null;
        this.theme = null;
        this.badge = null;
        // Outputs.
        this.resolved = new EventEmitter();
        // Data.
        this._widgetId = null;
        this._grecaptcha = null;
        this._executeRequested = false;
        // Form methods.
        this._onChange = null;
        this._onTouched = null;
        // Subscriptions.
        this._subscriptions = new Subscription();
        this.initializeProperties();
    }
    ngAfterViewInit() {
        const subscription = this._loader.ready$.subscribe((grecaptcha) => {
            if (grecaptcha == null || !(grecaptcha.render instanceof Function)) {
                return;
            }
            this._grecaptcha = grecaptcha;
            this.renderCaptchaElement();
        });
        this._subscriptions.add(subscription);
    }
    ngOnDestroy() {
        var _a;
        // Reset the captcha.
        this.resetCaptchaElement();
        // Unsubscribe of ready data.
        (_a = this._subscriptions) === null || _a === void 0 ? void 0 : _a.unsubscribe();
    }
    execute() {
        // Check if the captcha is invisible.
        if (this.size !== 'invisible') {
            return;
        }
        // Check if the widget exists else return.
        if (this._widgetId == null) {
            this._executeRequested = true;
            return;
        }
        // Execute the captcha.
        this._grecaptcha.execute(this._widgetId);
    }
    reset() {
        if (this._widgetId == null) {
            return;
        }
        if (this._grecaptcha.getResponse(this._widgetId)) {
            this.resolved.emit(null);
        }
        this.resetCaptchaElement();
    }
    writeValue(value) {
        if (value) {
            return;
        }
        this.reset();
    }
    registerOnChange(fn) {
        this._onChange = fn;
    }
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    onResolve(event) {
        var _a, _b;
        // Emit the value.
        this.resolved.emit(event);
        // Call the form functions.
        (_a = this._onChange) === null || _a === void 0 ? void 0 : _a.call(this, event);
        (_b = this._onTouched) === null || _b === void 0 ? void 0 : _b.call(this);
    }
    renderCaptchaElement() {
        this._widgetId = this._grecaptcha.render(this._elementRef.nativeElement, {
            badge: this.badge,
            sitekey: this.siteKey,
            size: this.size,
            tabindex: this.tabIndex,
            theme: this.theme,
            type: this.type,
            'callback': (response) => this._zone.run(() => this.onResolve(response)),
            'expired-callback': () => this._zone.run(() => this.onResolve(null))
        });
        if (this._executeRequested) {
            this._executeRequested = false;
            this.execute();
        }
    }
    resetCaptchaElement() {
        if (this._widgetId == null) {
            return;
        }
        this._zone.runOutsideAngular(() => this._grecaptcha.reset(this._widgetId));
    }
    initializeProperties() {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        this.siteKey = (_a = this.siteKey) !== null && _a !== void 0 ? _a : (_b = this._config) === null || _b === void 0 ? void 0 : _b.siteKey;
        this.theme = (_c = this.theme) !== null && _c !== void 0 ? _c : (_d = this._config) === null || _d === void 0 ? void 0 : _d.theme;
        this.size = (_e = this.size) !== null && _e !== void 0 ? _e : (_f = this._config) === null || _f === void 0 ? void 0 : _f.size;
        this.type = (_g = this.type) !== null && _g !== void 0 ? _g : (_h = this._config) === null || _h === void 0 ? void 0 : _h.type;
        this.badge = (_j = this.badge) !== null && _j !== void 0 ? _j : (_k = this._config) === null || _k === void 0 ? void 0 : _k.badge;
    }
}
RecaptchaComponent.decorators = [
    { type: Component, args: [{
                selector: 'bb-recaptcha',
                template: "",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                exportAs: 'bbRecaptcha',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => RecaptchaComponent),
                        multi: true
                    }
                ],
                host: {
                    'class': 'bb-recaptcha',
                    '[class.grouped]': 'grouped'
                },
                inputs: ['grouped'],
                preserveWhitespaces: false,
                styles: [".bb-recaptcha{display:block}.bb-recaptcha.grouped{margin-bottom:20px}"]
            },] }
];
RecaptchaComponent.ctorParameters = () => [
    { type: NgZone },
    { type: ElementRef },
    { type: RecaptchaLoader },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [RECAPTCHA_CONFIG,] }] }
];
RecaptchaComponent.propDecorators = {
    id: [{ type: HostBinding, args: ['attr.id',] }, { type: Input }],
    siteKey: [{ type: Input }],
    tabIndex: [{ type: Input }],
    type: [{ type: Input }],
    size: [{ type: Input }],
    theme: [{ type: Input }],
    badge: [{ type: Input }],
    resolved: [{ type: Output }]
};

class RecaptchaModule {
    static forRoot(config) {
        return {
            ngModule: RecaptchaModule,
            providers: [
                { provide: RECAPTCHA_CONFIG, useValue: config },
                RecaptchaLoader
            ]
        };
    }
}
RecaptchaModule.decorators = [
    { type: NgModule, args: [{
                declarations: [RecaptchaComponent],
                exports: [RecaptchaComponent]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { RECAPTCHA_CONFIG, RecaptchaComponent, RecaptchaLoader, RecaptchaModule };
//# sourceMappingURL=bravobit-bb-foundation-recaptcha.js.map
